/* 
   (c) Copyright 2001-2002  convergence integrated media GmbH.
   (c) Copyright 2002-2005  convergence GmbH.

   All rights reserved.

   Written by Denis Oliver Kropp <dok@directfb.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <pthread.h>
#include <directfb.h>

#include <direct/debug.h>
#include <direct/messages.h>

#include "util.h"

#include "box.h"
#include "window.h"

D_DEBUG_DOMAIN( Lite_Box, "LiTE/Box", "LiteBox");

static void append_child (LiteBox *box, LiteBox *child);

LiteBox *lite_new_box (LiteBox *parent, int x, int y, int width, int height)
{
     LiteBox          *box;

     if (!parent)
          return NULL;

     box = calloc (1, sizeof (LiteBox));

     box->parent = parent;

     box->rect.x = x;
     box->rect.y = y;
     box->rect.w = width;
     box->rect.h = height;

     if (lite_init_box (box)) {
          free (box);
          return NULL;
     }

     return box;
}

static void lite_draw_box_and_children (LiteBox *box, const DFBRegion *region, DFBBoolean clear)
{
     int i;

     if (region->x2 < region->x1 || region->y2 < region->y1)
          return;

     if (region->x1 > box->rect.w - 1 || region->x2 < 0 ||
         region->y1 > box->rect.h - 1 || region->y2 < 0)
          return;

     D_DEBUG( "Lite/Box: Drawing box at %4d,%4d - %4dx%4d\n",
              box->rect.x, box->rect.y, box->rect.w, box->rect.h );

     /* draw box */
     if (box->Draw)
          box->Draw( box, region, clear );

     /* draw children */
     for (i=0; i<box->n_children; i++) {
          DFBRegion  trans;
          LiteBox   *child = box->children[i];

          trans.x1 = region->x1 - child->rect.x;
          trans.y1 = region->y1 - child->rect.y;
          trans.x2 = region->x2 - child->rect.x;
          trans.y2 = region->y2 - child->rect.y;

          lite_draw_box_and_children( child, &trans, DFB_FALSE );
     }
}

DFBResult lite_draw_box (LiteBox *box, const DFBRegion *region, DFBBoolean flip)
{
     if (region) {
          D_DEBUG( "Lite/Box: Updating area at %4d,%4d - %4dx%4d\n", region->x1, region->y1,
                   region->x2 - region->x1 + 1, region->y2 - region->y1 + 1 );

          lite_draw_box_and_children( box, region, DFB_TRUE );

          if (flip)
               box->surface->Flip( box->surface, region, 0 );
     }
     else {
          DFBRegion reg = { 0, 0, box->rect.x + box->rect.w - 1, box->rect.y + box->rect.h - 1 };

          lite_draw_box( box, &reg, flip );
     }

     return DFB_OK;
}

DFBResult lite_update_box (LiteBox         *box,
                      const DFBRegion *region)
{
     if (region) {
          if (region->x2 < region->x1 || region->y2 < region->y1)
               return DFB_OK;

          if (region->x1 > box->rect.w - 1 || region->x2 < 0 ||
              region->y1 > box->rect.h - 1 || region->y2 < 0)
               return DFB_OK;

          if (box->parent) {
               DFBRegion trans;

               trans.x1 = region->x1 + box->rect.x;
               trans.y1 = region->y1 + box->rect.y;
               trans.x2 = region->x2 + box->rect.x;
               trans.y2 = region->y2 + box->rect.y;

               lite_update_box (box->parent, &trans);
          }
          else if (box->type == LITE_TYPE_WINDOW)
               lite_update_window( LITE_WINDOW (box), region );
          else
               D_ERROR( "Lite/Box: Can't update a box without a top level parent!\n" );
     }
     else {
          DFBRegion reg = { 0, 0, box->rect.w - 1, box->rect.h - 1 };

          lite_update_box( box, &reg );
     }

     return DFB_OK;
}

void lite_destroy_box (LiteBox *box)
{
     int i;

     /* destroy children */
     for (i=0; i<box->n_children; i++)
          box->children[i]->Destroy (box->children[i]);

     box->surface->Release (box->surface);

     free (box->children);
     free (box);
}

/* lite internals */

DFBResult lite_init_box (LiteBox *box)
{
     DFBResult ret;

     /* Get sub surface */
     ret = box->parent->surface->GetSubSurface (box->parent->surface,
                                                &box->rect, &box->surface);
     if (ret) {
          DirectFBError ("lite_init_box: Surface->GetSubSurface", ret);
          return -1;
     }

     box->Destroy = lite_destroy_box;

     append_child (box->parent, box);

     return ret;
}

DFBResult lite_reinit_box_and_children (LiteBox *box)
{
     int i;
     DFBResult ret = DFB_OK;

     /* Get new sub surface */
     if (box->parent) {
          IDirectFBSurface *surface;

          ret = box->parent->surface->GetSubSurface (box->parent->surface,
                                                     &box->rect, &surface);
          if (ret) {
               DirectFBError ("lite_reinit_box: Surface->GetSubSurface", ret);
               return ret;
          }

          ret = box->surface->Release( box->surface );

          box->surface = surface;
     }

     /* reinit children */
     for (i=0; i<box->n_children; i++)
          ret = lite_reinit_box_and_children( box->children[i] );

    return ret;
}

DFBResult lite_clear_box (LiteBox *box, const DFBRegion *region)
{
     if (box->parent) {
          DFBRegion reg;

          if (region) {
               reg = *region;

               reg.x1 += box->rect.x;
               reg.x2 += box->rect.x;
               reg.y1 += box->rect.y;
               reg.y2 += box->rect.y;
          }
          else {
               reg.x1 = box->rect.x;
               reg.y1 = box->rect.y;
               reg.x2 = box->rect.x + box->rect.w - 1;
               reg.y2 = box->rect.y + box->rect.h - 1;
          }

          if (box->parent->Draw)
               box->parent->Draw (box->parent, &reg, DFB_TRUE);
          else
               lite_clear_box (box->parent, &reg);
     }
     else {
          D_DEBUG_AT ( Lite_Box, "no parent!\n");
     }
    return DFB_OK;
}

/* file internals */

static void append_child (LiteBox *box, LiteBox *child)
{
     box->n_children++;

     box->children = realloc (box->children,
                              sizeof (LiteBox*) * box->n_children);

     box->children[box->n_children-1] = child;
}
