/* 
   (c) Copyright 2001-2002  convergence integrated media GmbH.
   (c) Copyright 2002-2005  convergence GmbH.

   All rights reserved.

   Written by Denis Oliver Kropp <dok@directfb.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef __LITE__WINDOW_H__
#define __LITE__WINDOW_H__

#include <directfb.h>

#include "box.h"

#define LITE_WINDOW(l) ((LiteWindow*)(l))

typedef enum {
     LITE_BLEND_ALWAYS,
     LITE_BLEND_NEVER,
     LITE_BLEND_AUTO
} LiteBlendMode;

enum {
     LITE_CENTER_HORIZONTALLY = -1,
     LITE_CENTER_VERTICALLY = -2
};

typedef void (*WindowIdleFunc) (void *data);
typedef void (*WindowMouseFunc) (DFBWindowEvent* evt, void *data);
typedef void (*WindowKeyboardFunc) (DFBWindowEvent* evt, void *data);

typedef struct _LiteWindow {
  LiteBox           box;

  int               width;
  int               height;

  DFBWindowID       id;

  char                 *title;
  IDirectFBWindow      *window;
  IDirectFBSurface     *surface;
  IDirectFBEventBuffer *events;

  int               moving;
  int               resizing;
  int               old_x;
  int               old_y;

  int               step_x;
  int               step_y;

  int               min_width;
  int               min_height;

  int               last_width;
  int               last_height;

  DFBWindowEvent    last_resize;
  DFBWindowEvent    last_motion;

  struct timeval    last_click;

  int               has_focus;

  WindowIdleFunc     idle_func;
  void              *idle_data;
  WindowMouseFunc    mouse_func;
  void              *mouse_data;
  WindowKeyboardFunc keyboard_func;
  void              *keyboard_data;

  struct {
       DFBRegion    region;
       DFBBoolean   pending;
  } update;

  LiteBlendMode     content_mode;
  LiteBlendMode     opacity_mode;

  struct {
       DFBBoolean   enabled;
       DFBColor     color;
  } bg;

  LiteBox          *entered_box;
  LiteBox          *focused_box;

  LiteBox          *drag_box;

  int (*OnMove)     (struct _LiteWindow *self, int x, int y);
  int (*OnResize)   (struct _LiteWindow *self, int width, int height);

  int (*OnClose)    (struct _LiteWindow *self);
  int (*OnDestroy)  (struct _LiteWindow *self);

  int (*OnFocusIn)  (struct _LiteWindow *self);
  int (*OnFocusOut) (struct _LiteWindow *self);

  int (*OnEnter)    (struct _LiteWindow *self, int x, int y);
  int (*OnLeave)    (struct _LiteWindow *self, int x, int y);
} LiteWindow;



LiteWindow *lite_new_window            (IDirectFBDisplayLayer  *layer,
                                        int                     x,
                                        int                     y,
                                        int                     width,
                                        int                     height,
                                        DFBWindowCapabilities   caps,
                                        const char             *title);

DFBResult   lite_window_event_loop     (LiteWindow             *window,
                                        int                     timeout);

DFBResult   lite_update_window         (LiteWindow             *window,
                                        const DFBRegion        *region);

DFBResult   lite_set_window_title      (LiteWindow             *window,
                                        const char             *title);

DFBResult   lite_set_window_opacity    (LiteWindow             *window,
                                        __u8                    opacity);

DFBResult   lite_set_window_background (LiteWindow             *window,
                                        DFBColor               *color);

DFBResult   lite_set_window_background_color (LiteWindow             *window,
                                              __u8                    r,
                                              __u8                    g,
                                              __u8                    b,
                                              __u8                    a);

DFBResult   lite_set_window_blend_mode (LiteWindow             *window,
                                        LiteBlendMode           content,
                                        LiteBlendMode           opacity);

DFBResult   lite_resize_window         (LiteWindow             *window,
                                        unsigned int            width,
                                        unsigned int            height);

DFBResult   lite_minimize_window       (LiteWindow             *window);

DFBResult   lite_restore_window        (LiteWindow             *window);

DFBResult   lite_focus_box             (LiteBox                *box);

int         lite_handle_window_event   (LiteWindow             *window,
                                        DFBWindowEvent         *event);

DFBResult   lite_flush_window_events   (LiteWindow             *window);

LiteWindow *lite_find_my_window        (LiteBox                *box);

DFBResult   lite_close_window          (LiteWindow             *window);

DFBResult   lite_destroy_window        (LiteWindow             *window);

int         lite_on_window_idle        (LiteWindow *window, 
                                        WindowIdleFunc, 
                                        void *data);

int         lite_on_window_mouse       (LiteWindow *window, 
                                        WindowMouseFunc, 
                                        void *data);

int         lite_on_window_keyboard    (LiteWindow *window, 
                                        WindowKeyboardFunc, 
                                        void *data);

#endif /*  __LITE__WINDOW_H__  */
